@@ -23,11 +23,12 @@
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
-version = '0.5.4'
+version = '0.5.5'
 
 import re
 import socket
 import functools
+import itertools
 import warnings
 import logging
 import traceback
@@ -75,12 +76,32 @@ def utf8(s):
     return str(s)
 
 
+def parse_requestline(s):
+    """
+    http://www.w3.org/Protocols/rfc2616/rfc2616-sec5.html#sec5
+
+    >>> parse_requestline('GET / HTTP/1.0')
+    ('GET', '/', '1.0')
+    >>> parse_requestline('post /testurl htTP/1.1')
+    ('POST', '/testurl', '1.1')
+    >>> parse_requestline('Im not a RequestLine')
+    Traceback (most recent call last):
+        ...
+    ValueError: Not a Request-Line
+    """
+    methods = '|'.join(HTTPretty.METHODS)
+    m = re.match(r'('+methods+')\s+(.*)\s+HTTP/(1.[0|1])', s, re.I)
+    if m:
+        return m.group(1).upper(), m.group(2), m.group(3)
+    else:
+        raise ValueError('Not a Request-Line')
+
+
 class HTTPrettyRequest(BaseHTTPRequestHandler, object):
     def __init__(self, headers, body=''):
         self.body = utf8(body)
         self.raw_headers = utf8(headers)
-
-        self.rfile = StringIO(headers + body)
+        self.rfile = StringIO('\r\n\r\n'.join([headers.strip(), body]))
         self.raw_requestline = self.rfile.readline()
         self.error_code = self.error_message = None
         self.parse_request()
@@ -111,10 +132,6 @@ class FakeSockFile(StringIO):
             new_amount = self.len - self.tell()
 
         ret = StringIO.read(self, new_amount)
-        remaining = amount - new_amount - 1
-        if remaining > 0:
-            ret = ret + (" " * remaining)
-
         return ret
 
 
@@ -212,7 +229,8 @@ class fakesock(object):
             hostnames = [i.hostname for i in HTTPretty._entries.keys()]
             self.fd.seek(0)
             try:
-                verb, headers_string = data.split('\n', 1)
+                requestline, _ = data.split('\r\n', 1)
+                method, path, version = parse_requestline(requestline)
                 is_parsing_headers = True
             except ValueError:
                 is_parsing_headers = False
@@ -224,17 +242,16 @@ class fakesock(object):
                 if len(self._sent_data) > 1:
                     headers, body = map(utf8, self._sent_data[-2:])
                     try:
-                        return HTTPretty.historify_request(headers, body)
+                        return HTTPretty.historify_request(headers, body, False)
 
                     except Exception, e:
                         logging.error(traceback.format_exc(e))
                         return self._true_sendall(data, *args, **kw)
 
-            method, path, version = re.split('\s+', verb.strip(), 3)
             # path might come with
             s = urlsplit(path)
 
-            headers, body = map(utf8, data.split('\r\n\r\n'))
+            headers, body = map(utf8, data.split('\r\n\r\n', 1))
 
             request = HTTPretty.historify_request(headers, body)
 
@@ -485,7 +502,8 @@ class Entry(object):
         fk.write('\n\r\n')
 
         if self.streaming:
-            for chunk in self.body:
+            self.body, body = itertools.tee(self.body)
+            for chunk in body:
                 fk.write(utf8(chunk))
         else:
             fk.write(utf8(self.body))
@@ -581,13 +599,23 @@ class HTTPretty(object):
     DELETE = 'DELETE'
     HEAD = 'HEAD'
     PATCH = 'PATCH'
+    METHODS = (GET, PUT, POST, DELETE, HEAD, PATCH)
     last_request = HTTPrettyRequestEmpty()
 
     @classmethod
-    def historify_request(cls, headers, body=''):
+    def reset(cls):
+        cls._entries.clear()
+        cls.latest_requests = []
+        cls.last_request = HTTPrettyRequestEmpty()
+
+    @classmethod
+    def historify_request(cls, headers, body='', append=True):
         request = HTTPrettyRequest(headers, body)
         cls.last_request = request
-        cls.latest_requests.append(request)
+        if append:
+            cls.latest_requests.append(request)
+        else:
+            cls.latest_requests[-1] = request
         return request
 
     @classmethod
@@ -706,12 +734,10 @@ def httprettified(test):
     "A decorator tests that use HTTPretty"
     @functools.wraps(test)
     def wrapper(*args, **kw):
+        HTTPretty.reset()
         HTTPretty.enable()
-        HTTPretty._entries.clear()
-
         try:
             return test(*args, **kw)
         finally:
             HTTPretty.disable()
-
     return wrapper
